(* Kalender drucken mit Vorschau

    2013-2024 J. Rathlev 24222 Schwentinental
      Web:  www.rathlev-home.de
      Mail: info(a)rathlev-home.de

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
   letzte nderung:  Jan. 2024
   *)

unit PrintPreviewDlg;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms, Vcl.Controls,
  Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls, Vcl.Dialogs, Vcl.ComCtrls,
  WinPlot, WinUtils, DateUtilEx, NumberEd, System.ImageList, Vcl.ImgList;

const
  VRand : TFRect = (Left : 1.5; Top : 1.0; Right : 0.5; Bottom : 1.0);
  HRand : TFRect = (Left : 0.5; Top : 1.0; Right : 0.5; Bottom : 0.5);
  defPoSize : TFPoint = (X : 21; Y : 29.6);
  defLaSize : TFPoint = (X : 29.6; Y : 21);

type
  TPrintPreviewDialog = class(TForm)
    pnlBottom: TPanel;
    btnPrint: TBitBtn;
    btnClose: TBitBtn;
    pnlMain: TPanel;
    pbxCal: TPaintBox;
    btnPrinterSelect: TBitBtn;
    PrinterSetupDialog: TPrinterSetupDialog;
    StatusBar: TStatusBar;
    PrintDialog: TPrintDialog;
    pnlSettings: TPanel;
    rgLayout: TRadioGroup;
    gbColors: TGroupBox;
    pnlSuCol: TPanel;
    ColorDialog: TColorDialog;
    btnSuCol: TBitBtn;
    btnSaCol: TBitBtn;
    pnlSaCol: TPanel;
    btnHoCol: TBitBtn;
    pnlHoCol: TPanel;
    btnDtCol: TBitBtn;
    pnlDtCol: TPanel;
    cbShowDates: TCheckBox;
    cbShowNames: TCheckBox;
    gbView: TGroupBox;
    cbShowSat: TCheckBox;
    pnlPaint: TPanel;
    laPage: TLabel;
    cbShowWeeks: TCheckBox;
    cbShowPriv: TCheckBox;
    pnlPrCol: TPanel;
    btnPrCol: TBitBtn;
    pnlWidth: TPanel;
    Label1: TLabel;
    rbFull: TRadioButton;
    reWidth: TFloatRangeEdit;
    Label2: TLabel;
    rbManual: TRadioButton;
    cbShowHidden: TCheckBox;
    pnlHdCol: TPanel;
    btnHdCol: TBitBtn;
    bbMargin: TBitBtn;
    pnlPage: TPanel;
    rePage: TRangeEdit;
    udpage: TNumUpDown;
    ilMoonPhases: TImageList;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnPrinterSelectClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure pbxCalPaint(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure btnPrintClick(Sender: TObject);
    procedure UpdateView(Sender: TObject);
    procedure pnlSuColClick(Sender: TObject);
    procedure pnlSaColClick(Sender: TObject);
    procedure pnlHoColClick(Sender: TObject);
    procedure pnlDtColClick(Sender: TObject);
    procedure rgLayoutClick(Sender: TObject);
    procedure btnSuColClick(Sender: TObject);
    procedure btnSaColClick(Sender: TObject);
    procedure btnHoColClick(Sender: TObject);
    procedure btnDtColClick(Sender: TObject);
    procedure pnlPrColClick(Sender: TObject);
    procedure btnPrColClick(Sender: TObject);
    procedure rbFullClick(Sender: TObject);
    procedure rbManualClick(Sender: TObject);
    procedure reWidthExit(Sender: TObject);
    procedure pnlHdColClick(Sender: TObject);
    procedure btnHdColClick(Sender: TObject);
    procedure bbMarginClick(Sender: TObject);
    procedure FormMouseWheelDown(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure FormMouseWheelUp(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
  private
    { Private-Deklarationen }
    FIniName,
    PrtName       : string;
    PvPlot,PrPlot : TPlot;
    PrSize        : TFPoint;  { Papiergre in cm/inch }
    Rand          : TFRect;   { Rnder in cm/inch }
    PvSize        : TPoint;   { Bildschirmgrafikfenster in Pixel }
    FYear         : integer;
    NoPrinter     : boolean;
    SuFCol,SaFCol,
    HoFCol,DtFCol,
    PrFCol,PrCol,
    HdFCol,HdCol,
    SuCol,SaCol,
    HoCol,DtCol   : TColor;
    FHolidayCalendar : THolidayCalendar;
    function ChangeColor (var AColor : TColor) : boolean;
    procedure SetLayout;
    procedure NewSize;
    procedure PrintCalendar(APlot : TPlot; Page : integer);
  public
    { Public-Deklarationen }
    procedure LoadFromIni (const AIniName : string);
    procedure Execute (AYear : integer; AHolidayCalendar : THolidayCalendar);
  end;

var
  PrintPreviewDialog: TPrintPreviewDialog;

implementation


{$R *.dfm}

uses Printers, IniFiles, DateUtils, RandDlg;

const
  clLightRed  = $87D0FF;
  clLightGreen = $0084FBC8;
  clLightYellow = $009FF4FB;
  clLightBlue = $00FFDB91;
  clLightMag = $00CF9FFF;
  clLightSteel = $00EEE1C8;

  PrtSekt = 'Printer';
  PoSekt  = 'PrintPortrait';
  LsSekt  = 'PrintLandscape';

  iniPrtName = 'Name';
  iniTop  = 'Top';
  iniLeft = 'Left';
  iniRight = 'Right';
  iniBottom = 'Bottom';
  iniWidth = 'Width';
  iniHeight = 'Height';
  iniHoCol = 'HolidayColor';
  iniDtCol = 'DatesColor';
  iniSuCol = 'SundayColor';
  iniSaCol = 'SaturdayColor';
  iniPrCol = 'PrivateColor';
  iniHdCol = 'HiddenColor';
  iniHoFCol = 'HolidayFontColor';
  iniDtFCol = 'DatesFontColor';
  iniSuFCol = 'SundayFontColor';
  iniSaFCol = 'SaturdayFontColor';
  iniPrFCol = 'PrivateFontColor';
  iniHdFCol = 'HiddenFontColor';
  iniShHol = 'ShowHolidayNames';
  iniShDts = 'ShowDates';
  iniShSat = 'ShowSaturdays';
  iniShWk  = 'ShowWeeks';
  iniShHid = 'ShowHidden';
  iniShPrv = 'ShowPrivate';
  iniLayout = 'Layout';
  iniPWidth = 'PrintWidth';

procedure TPrintPreviewDialog.FormCreate(Sender: TObject);
begin
  PvPlot:=TPlot.Create(pbxCal.Canvas);
  FIniName:='';
  PrSize:=defLaSize;
  SuCol:=clLightRed;
  SaCol:=clLightBlue;
  HoCol:=clLightGreen;
  DtCol:=clLightYellow;
  PrCol:=clLightMag;
  HdCol:=clLightSteel;
  SuFCol:=clBlack;
  SaFCol:=clBlack;
  HoFCol:=clBlack;
  DtFCol:=clBlack;
  PrFCol:=clBlack;
  HdFCol:=clBlack;
  cbShowNames.Checked:=true;
  cbShowDates.Checked:=true;
  cbShowSat.Checked:=true;
  cbShowPriv.Checked:=true;
  cbShowHidden.Checked:=false;
  cbShowWeeks.Checked:=false;
  reWidth.Value:=12;
  NoPrinter:=true;
  end;

procedure TPrintPreviewDialog.LoadFromIni (const AIniName : string);
begin
  if FileExists(AIniName) then begin
    FIniName:=AIniName;
    with TMemIniFile.Create(AIniName) do begin
      PrtName:=ReadString(PrtSekt,iniPrtName,'');
      ClientWidth:=ReadInteger(PrtSekt,iniWidth,ClientWidth);
      ClientHeight:=ReadInteger(PrtSekt,iniHeight,ClientHeight);
      rgLayOut.ItemIndex:=ReadInteger(PrtSekt,iniLayout,0);
      SuCol:=ReadInteger(PrtSekt,iniSuCol,clLightRed);
      SaCol:=ReadInteger(PrtSekt,iniSaCol,clLightBlue);
      HoCol:=ReadInteger(PrtSekt,iniHoCol,clLightGreen);
      DtCol:=ReadInteger(PrtSekt,iniDtCol,clLightYellow);
      PrCol:=ReadInteger(PrtSekt,iniPrCol,clLightMag);
      HdCol:=ReadInteger(PrtSekt,iniHdCol,clWhite);
      SuFCol:=ReadInteger(PrtSekt,iniSuFCol,clRed);
      SaFCol:=ReadInteger(PrtSekt,iniSaFCol,clBlack);
      HoFCol:=ReadInteger(PrtSekt,iniHoFCol,clRed);
      DtFCol:=ReadInteger(PrtSekt,iniDtFCol,clBlack);
      PrFCol:=ReadInteger(PrtSekt,iniPrFCol,clBlack);
      HdFCol:=ReadInteger(PrtSekt,iniHdFCol,clBlack);
      cbShowNames.Checked:=ReadBool(PrtSekt,iniShHol,true);
      cbShowDates.Checked:=ReadBool(PrtSekt,iniShDts,true);
      cbShowSat.Checked:=ReadBool(PrtSekt,iniShSat,true);
      cbShowPriv.Checked:=ReadBool(PrtSekt,iniShPrv,true);
      cbShowHidden.Checked:=ReadBool(PrtSekt,iniShHid,false);
      cbShowWeeks.Checked:=ReadBool(PrtSekt,iniShWk,false);
      reWidth.Value:=ReadInteger(PrtSekt,iniPWidth,120)/10;
      with HRand do begin
        Top:=ReadInteger(LsSekt,iniTop,round(10*Top))/10;
        Left:=ReadInteger(LsSekt,iniTop,round(10*Left))/10;
        Bottom:=ReadInteger(LsSekt,iniTop,round(10*Bottom))/10;
        Right:=ReadInteger(LsSekt,iniTop,round(10*Right))/10;
        end;
      with VRand do begin
        Top:=ReadInteger(PoSekt,iniTop,round(10*Top))/10;
        Left:=ReadInteger(PoSekt,iniTop,round(10*Left))/10;
        Bottom:=ReadInteger(PoSekt,iniTop,round(10*Bottom))/10;
        Right:=ReadInteger(PoSekt,iniTop,round(10*Right))/10;
        end;
      end;
    end;
  end;

procedure TPrintPreviewDialog.FormDestroy(Sender: TObject);
begin
  PvPlot.Free; PrPlot.Free;
  if length(FIniName)>0 then with TMemIniFile.Create(FIniName) do begin
    WriteString (PrtSekt,iniPrtName,PrtName);
    WriteInteger(PrtSekt,iniWidth,ClientWidth);
    WriteInteger(PrtSekt,iniHeight,ClientHeight);
    WriteInteger(PrtSekt,iniLayout,rgLayOut.ItemIndex);
    WriteInteger(PrtSekt,iniSuCol,SuCol);
    WriteInteger(PrtSekt,iniSaCol,SaCol);
    WriteInteger(PrtSekt,iniHoCol,HoCol);
    WriteInteger(PrtSekt,iniDtCol,DtCol);
    WriteInteger(PrtSekt,iniPrCol,PrCol);
    WriteInteger(PrtSekt,iniHdCol,HdCol);
    WriteInteger(PrtSekt,iniSuFCol,SuFCol);
    WriteInteger(PrtSekt,iniSaFCol,SaFCol);
    WriteInteger(PrtSekt,iniHoFCol,HoFCol);
    WriteInteger(PrtSekt,iniDtFCol,DtFCol);
    WriteInteger(PrtSekt,iniPrFCol,PrFCol);
    WriteInteger(PrtSekt,iniHdFCol,HdFCol);
    WriteBool(PrtSekt,iniShHol,cbShowNames.Checked);
    WriteBool(PrtSekt,iniShDts,cbShowDates.Checked);
    WriteBool(PrtSekt,iniShSat,cbShowSat.Checked);
    WriteBool(PrtSekt,iniShPrv,cbShowPriv.Checked);
    WriteBool(PrtSekt,iniShHid,cbShowHidden.Checked);
    WriteBool(PrtSekt,iniShWk,cbShowWeeks.Checked);
    WriteInteger(PrtSekt,iniPWidth,round(10*reWidth.Value));
    with HRand do begin
      WriteInteger(LsSekt,iniTop,round(10*Top));
      WriteInteger(LsSekt,iniTop,round(10*Left));
      WriteInteger(LsSekt,iniTop,round(10*Bottom));
      WriteInteger(LsSekt,iniTop,round(10*Right));
      end;
    with VRand do begin
      WriteInteger(PoSekt,iniTop,round(10*Top));
      WriteInteger(PoSekt,iniTop,round(10*Left));
      WriteInteger(PoSekt,iniTop,round(10*Bottom));
      WriteInteger(PoSekt,iniTop,round(10*Right));
      end;
    UpdateFile;
    Free;
    end;
  end;

procedure TPrintPreviewDialog.FormMouseWheelDown(Sender: TObject;
  Shift: TShiftState; MousePos: TPoint; var Handled: Boolean);
begin
  Handled:=true;
  with rePage do if Value<MaxValue then begin
    Value:=Value+1;
    pbxCal.Invalidate;
    end;
  end;

procedure TPrintPreviewDialog.FormMouseWheelUp(Sender: TObject;
  Shift: TShiftState; MousePos: TPoint; var Handled: Boolean);
begin
  Handled:=true;
  with rePage do if Value>0 then begin
    Value:=Value-1;
    pbxCal.Invalidate;
    end;
  end;

procedure TPrintPreviewDialog.FormShow(Sender: TObject);
begin
  NoPrinter:=true;
  with Printer do begin
    if Printers.Count>0 then begin
      PrinterIndex:=Printers.IndexOf(PrtName);
      PrtName:=Printers[PrinterIndex];
      if PrinterIndex>=0 then begin
        Orientation:=poLandscape;
        btnPrint.Enabled:=true;
        btnPrinterSelect.Enabled:=true;
        PrintDialog.Copies:=1;
        with PrSize do begin
          X:=GetPaperWidth(Printer)/10;
          Y:=GetPaperHeight(Printer)/10;
          end;
        StatusBar.SimpleText:='Drucken auf: '+PrtName;
        PrPlot:=TPlot.Create (Canvas);
        NoPrinter:=false;
        end
      end;
    if NoPrinter then begin
      PrtName:='';
      btnPrint.Enabled:=false;
      btnPrinterSelect.Enabled:=false;
      PrSize:=defLaSize;
      StatusBar.SimpleText:='kein Drucker verfgbar!';
      end;
    end;
  with pnlSuCol do begin
    ParentBackground:=false;
    Color:=SuCol;
    Font.Color:=SuFCol;
    end;
  with pnlSaCol do begin
    ParentBackground:=false;
    Color:=SaCol;
    Font.Color:=SaFCol;
    end;
  with pnlHoCol do begin
    ParentBackground:=false;
    Color:=HoCol;
    Font.Color:=HoFCol;
    end;
  with pnlDtCol do begin
    ParentBackground:=false;
    Color:=DtCol;
    Font.Color:=DtFCol;
    end;
  with pnlPrCol do begin
    ParentBackground:=false;
    Color:=PrCol;
    Font.Color:=PrFCol;
    end;
  with pnlHdCol do begin
    ParentBackground:=false;
    Color:=HdCol;
    Font.Color:=HdFCol;
    end;
  SetLayout;
  end;

procedure TPrintPreviewDialog.FormResize(Sender: TObject);
var
  fp,fv : double;
begin
  if not Visible then Exit;
  with PrSize,Rand do fp:=(x-Left-Right)/(y-Top-Bottom);
  with pbxCal do fv:=Width/Height;
  with PvSize do begin
    if fp>=fv then begin
      X:=pbxCal.Width; Y:=round(X/fp);
      end
    else begin
      Y:=pbxCal.Height; X:=round(Y*fp);
      end;
    end;
  end;

procedure TPrintPreviewDialog.NewSize;
begin
  FormResize(self);
  PvPlot.SetPlotField (PrSize,Rand);
  pbxCal.Invalidate;
  end;

procedure TPrintPreviewDialog.btnPrinterSelectClick(Sender: TObject);
begin
  with Printer do if not Printing then begin
    if PrinterSetupDialog.Execute then begin
      Orientation:=poLandscape;
      PrtName:=Printers[PrinterIndex];
      StatusBar.SimpleText:='Drucken auf: '+PrtName;
      SetLayout;
      end;
    end;
  end;

procedure TPrintPreviewDialog.btnPrintClick(Sender: TObject);
var
  i : integer;
begin
  with Printer do begin
    Title:='Jahreskalender '+IntToStr(FYear);
    BeginDoc;
    with PrPlot do begin
      with PrSize do GetPaperSize (Printer,X,Y);
      SetPlotField (PrSize,Rand);
      for i:=1 to rePage.MaxValue do begin
        if i>1 then NewPage;
        SetPrintWindow (shMargin);
        PrintCalendar(PrPlot,i);
        end;
      end;
    EndDoc;
    end;
  end;

procedure TPrintPreviewDialog.pbxCalPaint(Sender: TObject);
begin
  if Visible then begin
    PvPlot.SetScreenWindow (shNoMargin,PvSize);
    PrintCalendar(PvPlot,rePage.Value);
    end;
  end;

procedure TPrintPreviewDialog.PrintCalendar(APlot : TPlot; Page : integer);
const
  TopHeight = 1.2;
  MonHeight = 0.75;
var
  CalWidth,CalHeight,
  mw,mh,xd,yd,xm,ym,d  : double;
  i,j,k,k1,n,m,dow   : integer;
  dt                 : TDateTime;
  hd,hw              : boolean;
  hv                 : THolidayViews;
  hday               : THoliday;
  bcol,fcol          : TColor;
  s1,s2,s            : string;
  bm                 : TBitmap;
begin
  with APlot do begin
    StartPlot(true,white);                // Zeichenfeld lschen, Farbe wei
    with Rand do
      CPlot(Left,Bottom,pmUpOrigin,mmNoMark);    { Nullpunkt unten links }
    with PlotField do begin
      CalWidth:=Right-Left;
      CalHeight:=Top-Bottom-TopHeight;
      end;
    bm:=TBitmap.Create;
    SetTextFont('Tahoma');
    SetTransparent(true);
    hv:=hvAll;
    if not cbShowPriv.Checked then exclude(hv,hvPrivate);
    if not cbShowDates.Checked then exclude(hv,hvOther);
    if not cbShowHidden.Checked then exclude(hv,hvHidden);
    if rgLayout.ItemIndex=3 then begin      // 1 Monat pro Seite
      // berschrift
      SetTextHeight(0.8); SetTextStyle(tsBold);
      PlotColStr(0,CalHeight+0.5,FormatSettings.LongMonthNames[Page]+' '+IntToStr(FYear),true,alLeftBottom,clBlack);
      SetTextHeight(0.5);
      mw:=CalWidth;
      mh:=CalHeight/31;
      xd:=0.7; yd:=CalHeight-mh/2;
      m:=Page;
      SetTextStyle(tsNormal);
      for j:=1 to DaysInMonth(EncodeDate(FYear,m,1)) do begin
        dt:=EncodeDate(FYear,m,j);
        dow:=DayOfWeek(dt);
        fcol:=clBlack;
        with FHolidayCalendar do begin
          k:=FindNextMatchingHoliday (dt,0,hv);
          hd:=(k>=0) and Holiday[GetHolidayIndex(k)].Active;
          if hd then begin
            bcol:=clWhite; fcol:=clBlack;
            hw:=Holiday[GetHolidayIndex(k)].HType=htWeek;   // wiederk. wchentlicher Termin
            k1:=k;
            repeat
              k1:=FindNextMatchingHoliday (dt,k1+1,hv);
              if k1>=0 then with Holiday[GetHolidayIndex(k1)] do begin
                hw:=hw or (HType=htWeek);
                if (HView<>hvHidden) and (hView<>hvNone) and (hView<>hvMoon) then k:=k1;
                end;
              until k1<0;
            hday:=Holiday[GetHolidayIndex(k)];
            with hday do begin
              if HView=hvHoliday then begin
                bcol:=HoCol; fcol:=HoFCol;
                end
              else begin
                if HView=hvPrivate then begin
                  bcol:=PrCol; fcol:=PrFCol;
                  end
                else if dow=1 then begin
                  bcol:=SuCol; fcol:=SuFCol;
                  end
                else if (dow=7) and cbShowSat.Checked then begin
                  bcol:=SaCol; fcol:=SaFCol;
                  end
                else if HView=hvOther then begin
                  bcol:=DtCol; fcol:=DtFCol;
                  end
                else if HView=hvHidden then begin
                  bcol:=HdCol; fcol:=HdFCol;
                  end;
                if HView=hvPrivate then hd:=cbShowPriv.Checked
                else if HView=hvOther then hd:=cbShowDates.Checked
                else if HView=hvHidden then hd:=cbShowHidden.Checked;
                end;
              end;
            end;
          end;
        if hd then begin  // Texte
          PlotRect(0,yd-mh*(j-1.5),mw,yd-mh*(j-0.5),bcol);
          s:=''; d:=0;
          with FHolidayCalendar do if ShowMoonCycles then begin
            n:=GetMoonPhase(dt);
            if (n>=0) then begin
              ilMoonPhases.GetBitmap(n,bm);
              PlotBitmap(2.5*xd,yd-mh*(j-0.5)+0.15,0.5*mh,0,bcol,ilMoonPhases.BkColor,bm);
              d:=0.5*mh;
              end
            end;
          if cbShowNames.Checked then begin
            SetTextHeight(0.35);
            PlotColStr(2.5*xd+d,yd-mh*(j-0.5)+0.2,FHolidayCalendar.GetHolidayNames(dt,htall-[htWeek],hv),
              true,alLeftBottom,clBlack);
            SetTextHeight(0.5);
            if hw then s:=FHolidayCalendar.GetHolidayNames(dt,[htWeek],hv);
            end;
          if (dow=2) and cbShowWeeks.Checked then begin
            s:=s+' '+IntToStr(WeekOfTheYear(dt));
            end;
          if length(s)>0 then PlotColStr(mw-0.2,yd-mh*(j-1),s,true,alRightCenter,clBlack);
          end
        else begin
          if dow=1 then begin
            PlotRect(0,yd-mh*(j-1.5),mw,yd-mh*(j-0.5),SuCol);
            fcol:=SuFCol;
            end
          else if dow=2 then begin  // Kalenderwoche am Montag anzeigen
            if cbShowWeeks.Checked then
              PlotColStr(mw-0.2,yd-mh*(j-1),IntToStr(WeekOfTheYear(dt)),
                true,alRightCenter,clBlack);
            end
          else if (dow=7) and cbShowSat.Checked then begin
            PlotRect(0,yd-mh*(j-1.5),mw,yd-mh*(j-0.5),SaCol);
            fcol:=SaFCol;
            end;
          end;
        PlotColStr(xd,yd-mh*(j-1),IntToStr(DayOfTheMonth(dt)),
          true,alRightCenter,fcol);
        PlotColStr(xd+0.2,yd-mh*(j-1),FormatSettings.ShortDayNames[dow],
          true,alLeftCenter,fcol);
        end;
      SetLineWidth(2);        // dnne Linien
      for j:=0 to 30 do begin
        CPlot (0,mh*j,3,0); CPlot (CalWidth,mh*j,2,0);
        end;
      SetLineWidth(5);        // dicke Linien
      CPlot (0,0,3,0); CPlot (0,CalHeight,2,0); CPlot (CalWidth,CalHeight,2,0);
      CPlot (CalWidth,0,2,0); CPlot (0,0,2,0);
      end
    else if rgLayout.ItemIndex=2 then begin // 2 Monate pro Seite
      // berschrift
      SetTextHeight(0.8); SetTextStyle(tsBold);
      s1:=FormatSettings.LongMonthNames[2*Page-1];
      s2:=FormatSettings.LongMonthNames[2*Page];
      PlotColStr(0,CalHeight+0.5,s1+' '+IntToStr(FYear),true,alLeftBottom,clBlack);
      PlotColStr(CalWidth/2+HRand.Left,CalHeight+0.5,s2+' '+IntToStr(FYear),true,alLeftBottom,clBlack);
      SetTextHeight(0.3);
      mw:=CalWidth/2-HRand.Left;
      mh:=CalHeight/31;
      yd:=CalHeight-mh/2;
      for i:=1 to 2 do begin
        xm:=(mw+2*HRand.Left)*(i-1);
        xd:=xm+0.4;
        m:=(Page-1)*2+i;
        SetTextStyle(tsNormal);
        for j:=1 to DaysInMonth(EncodeDate(FYear,m,1)) do begin
          dt:=EncodeDate(FYear,m,j);
          dow:=DayOfWeek(dt);
          fcol:=clBlack;
          with FHolidayCalendar do begin
            k:=FindNextMatchingHoliday(dt,0,hv);
            hd:=(k>=0) and Holiday[GetHolidayIndex(k)].Active;
            if hd then begin
              bcol:=clWhite; fcol:=clBlack;
              hw:=Holiday[GetHolidayIndex(k)].HType=htWeek;   // wiederk. wchentlicher Termin
              k1:=k;
              repeat
                k1:=FindNextMatchingHoliday(dt,k1+1,hv);
                if k1>=0 then with Holiday[GetHolidayIndex(k1)] do begin
                  hw:=hw or (HType=htWeek);
                  if (HView<>hvHidden) and (hView<>hvNone) and (hView<>hvMoon) then k:=k1;
                  end;
                until k1<0;
              with Holiday[GetHolidayIndex(k)] do begin
                if HView=hvHoliday then begin
                  bcol:=HoCol; fcol:=HoFCol;
                  end
                else begin
                  if HView=hvPrivate then begin
                    bcol:=PrCol; fcol:=PrFCol;
                    end
                  else if dow=1 then begin
                    bcol:=SuCol; fcol:=SuFCol;
                    end
                  else if (dow=7) and cbShowSat.Checked then begin
                    bcol:=SaCol; fcol:=SaFCol;
                    end
                  else if HView=hvPrivate then begin
                    bcol:=PrCol; fcol:=PrFCol;
                    end
                  else if HView=hvOther then begin
                    bcol:=DtCol; fcol:=DtFCol;
                    end
                  else if HView=hvHidden then begin
                    bcol:=HdCol; fcol:=HdFCol;
                    end;
                  if HView=hvPrivate then hd:=cbShowPriv.Checked
                  else if HView=hvOther then hd:=cbShowDates.Checked
                  else hd:=cbShowHidden.Checked;
                  end;
                end;
              end;
            end;
          if hd then begin
            PlotRect(xm,yd-mh*(j-1.5),xm+mw,yd-mh*(j-0.5),bcol);
            s:=''; d:=0;
            with FHolidayCalendar do if ShowMoonCycles then begin
              n:=FHolidayCalendar.GetMoonPhase(dt);
              if (n>=0) then begin
                ilMoonPhases.GetBitmap(n,bm);
                PlotBitmap(xd+0.6,yd-mh*(j-0.7),0.5*mh,0,bcol,ilMoonPhases.BkColor,bm);
                d:=0.5*mh;
                end
              end;
            if cbShowNames.Checked then begin
              SetTextHeight(0.2);
              PlotColStr(xd+d+0.6,yd-mh*(j-0.5)+0.15,FHolidayCalendar.GetHolidayNames(dt,htall-[htWeek],hv),
                true,alLeftBottom,clBlack);
              SetTextHeight(0.3);
              if hw then s:=FHolidayCalendar.GetHolidayNames(dt,[htWeek],hv);
              end;
            if (dow=2) and cbShowWeeks.Checked then begin
              s:=s+' '+IntToStr(WeekOfTheYear(dt));
              end;
            if length(s)>0 then PlotColStr(xm+mw-0.1,yd-mh*(j-1),s,true,alRightCenter,clBlack);
            end
          else begin
            if dow=1 then begin
              PlotRect(xm,yd-mh*(j-1.5),xm+mw,yd-mh*(j-0.5),SuCol);
              fcol:=SuFCol;
              end
            else if dow=2 then begin  // Kalenderwoche am Montag anzeigen
              if cbShowWeeks.Checked then
                PlotColStr(xm+mw-0.1,yd-mh*(j-1),IntToStr(WeekOfTheYear(dt)),
                  true,alRightCenter,clBlack);
              end
            else if (dow=7) and cbShowSat.Checked then begin
              PlotRect(xm,yd-mh*(j-1.5),xm+mw,yd-mh*(j-0.5),SaCol);
              fcol:=SaFCol;
              end;
            end;
          PlotColStr(xd,yd-mh*(j-1),IntToStr(DayOfTheMonth(dt)),
            true,alRightCenter,fcol);
          PlotColStr(xd+0.1,yd-mh*(j-1),FormatSettings.ShortDayNames[dow],
            true,alLeftCenter,fcol);
          end;
        SetLineWidth(2);        // dnne Linien
        for j:=0 to 30 do begin
          CPlot (xm,mh*j,3,0); CPlot (xm+mw,mh*j,2,0);
          end;
        SetLineWidth(5);        // dicke Linien
        CPlot (xm,0,3,0); CPlot (xm,CalHeight,2,0); CPlot (xm+mw,CalHeight,2,0);
        CPlot (xm+mw,0,2,0); CPlot (xm,0,2,0);
        end;
      end
    else if rgLayout.ItemIndex=1 then begin // 6 Monate pro Seite
      // berschrift
      SetTextHeight(0.8); SetTextStyle(tsBold);
      if Page=1 then s1:='1. Halbjahr ' else s1:='2. Halbjahr ';
      PlotColStr(0,CalHeight+0.5,s1+IntToStr(FYear),true,alLeftBottom,clBlack);
      mw:=CalWidth/6;
      mh:=(CalHeight-MonHeight)/31;
      xd:=0.4; yd:=CalHeight-MonHeight-mh/2;
      xm:=0.1; ym:=CalHeight-MonHeight/2;
      for i:=1 to 6 do begin
        m:=(Page-1)*6+i;
        SetTextHeight(0.4); SetTextStyle(tsBold);
        PlotColStr(mw*(i-1)+xm,ym,FormatSettings.LongMonthNames[m],true,alLeftCenter,clBlack);
        SetTextHeight(0.3); SetTextStyle(tsNormal);
        for j:=1 to DaysInMonth(EncodeDate(FYear,m,1)) do begin
          dt:=EncodeDate(FYear,m,j);
          dow:=DayOfWeek(dt);
          fcol:=clBlack;
          with FHolidayCalendar do begin
            k:=FindNextMatchingHoliday(dt,0,hv);
            hd:=(k>=0) and Holiday[GetHolidayIndex(k)].Active;
            if hd then begin
              bcol:=clWhite; fcol:=clBlack;
              hw:=Holiday[GetHolidayIndex(k)].HType=htWeek;   // wiederk. wchentlicher Termin
              k1:=k;
              repeat
                k1:=FindNextMatchingHoliday(dt,k1+1,hv);
                if k1>=0 then with Holiday[GetHolidayIndex(k1)] do begin
                  hw:=hw or (HType=htWeek);
                  if (HView<>hvHidden) and (hView<>hvNone) and (hView<>hvMoon) then k:=k1;
                  end;
                until k1<0;
              with Holiday[GetHolidayIndex(k)] do begin
                if HView=hvHoliday then begin
                  bcol:=HoCol; fcol:=HoFCol;
                  end
                else begin
                  if HView=hvPrivate then begin
                    bcol:=PrCol; fcol:=PrFCol;
                    end
                  else if dow=1 then begin
                    bcol:=SuCol; fcol:=SuFCol;
                    end
                  else if (dow=7) and cbShowSat.Checked then begin
                    bcol:=SaCol; fcol:=SaFCol;
                    end
                  else if HView=hvPrivate then begin
                    bcol:=PrCol; fcol:=PrFCol;
                    end
                  else if HView=hvOther then begin
                    bcol:=DtCol; fcol:=DtFCol;
                    end
                  else if HView=hvHidden then begin
                    bcol:=HdCol; fcol:=HdFCol;
                    end;
                  if HView=hvPrivate then hd:=cbShowPriv.Checked
                  else if HView=hvOther then hd:=cbShowDates.Checked
                  else hd:=cbShowHidden.Checked;
                  end;
                end;
              end;
            end;
          if hd then begin
            PlotRect(mw*(i-1),yd-mh*(j-1.5),mw*i,yd-mh*(j-0.5),bcol);
            s:=''; d:=0;
            with FHolidayCalendar do if ShowMoonCycles then begin
              n:=FHolidayCalendar.GetMoonPhase(dt);
              if (n>=0) then begin
                ilMoonPhases.GetBitmap(n,bm);
                PlotBitmap(mw*(i-1)+xd+0.5,yd-mh*(j-0.7),0.5*mh,0,bcol,ilMoonPhases.BkColor,bm);
                d:=0.4*mh;
                end
              end;
            SetTextHeight(0.25);
            if cbShowNames.Checked then s:=FHolidayCalendar.GetHolidayNames(dt,htall-[htWeek],hv);
            if hw then begin
              if length(s)>0 then s:=s+', ';
              s:=s+FHolidayCalendar.GetHolidayNames(dt,[htWeek],hv);
              end;
            if length(s)>0 then PlotColStr(mw*i-0.1,yd-mh*(j-1),s,true,alRightCenter,clBlack);
            SetTextHeight(0.3);
            end
          else begin
            if dow=1 then begin
              PlotRect(mw*(i-1),yd-mh*(j-1.5),mw*i,yd-mh*(j-0.5),SuCol);
              fcol:=SuFCol;
              end
            else if dow=2 then begin  // Kalenderwoche am Montag anzeigen
              if cbShowWeeks.Checked then
                PlotColStr(mw*i-0.1,yd-mh*(j-1),IntToStr(WeekOfTheYear(dt)),
                  true,alRightCenter,clBlack);
              end
            else if (dow=7) and cbShowSat.Checked then begin
              PlotRect(mw*(i-1),yd-mh*(j-1.5),mw*i,yd-mh*(j-0.5),SaCol);
              fcol:=SaFCol;
              end;
            end;
          PlotColStr(mw*(i-1)+xd,yd-mh*(j-1),IntToStr(DayOfTheMonth(dt)),
            true,alRightCenter,fcol);
          PlotColStr(mw*(i-1)+xd+0.1,yd-mh*(j-1),FormatSettings.ShortDayNames[dow],
            true,alLeftCenter,fcol);
          end;
        end;
      SetLineWidth(2);        // dnne Linien
      for j:=0 to 30 do begin
        CPlot (0,mh*j,3,0); CPlot (CalWidth,mh*j,2,0);
        end;
      SetLineWidth(5);        // dicke Linien
      CPlot (0,0,3,0); CPlot (0,CalHeight,2,0); CPlot (CalWidth,CalHeight,2,0);
      CPlot (CalWidth,0,2,0); CPlot (0,0,2,0);
      CPlot (0,CalHeight-MonHeight,3,0); CPlot (CalWidth,CalHeight-MonHeight,2,0);
      for i:=1 to 5 do begin
        CPlot (mw*i,0,3,0); CPlot (mw*i,CalHeight,2,0);
        end;
      end
    else begin                              // 12 Monate pro Seite
      // berschrift
      SetTextHeight(0.8); SetTextStyle(tsBold);
      PlotColStr(0,CalHeight+0.5,'Jahreskalender '+IntToStr(FYear),true,alLeftBottom,clBlack);
      SetTextHeight(0.3);
      mw:=CalWidth/12;
      mh:=(CalHeight-MonHeight)/31;
      xd:=0.4; yd:=CalHeight-MonHeight-mh/2;
      xm:=0.1; ym:=CalHeight-MonHeight/2;
      for i:=1 to 12 do begin
        SetTextStyle(tsBold);
        PlotColStr(mw*(i-1)+xm,ym,FormatSettings.LongMonthNames[i],true,alLeftCenter,clBlack);
        SetTextStyle(tsNormal);
        for j:=1 to DaysInMonth(EncodeDate(FYear,i,1)) do begin
          dt:=EncodeDate(FYear,i,j);
          dow:=DayOfWeek(dt);
          fcol:=clBlack;
          with FHolidayCalendar do begin
            k:=FindNextMatchingHoliday(dt,0,hv);
            hd:=(k>=0) and Holiday[GetHolidayIndex(k)].Active;
            if hd then begin
              bcol:=clWhite; fcol:=clBlack;
              hw:=Holiday[GetHolidayIndex(k)].HType=htWeek;   // wiederk. wchentlicher Termin
              k1:=k;
              repeat
                k1:=FindNextMatchingHoliday(dt,k1+1,hv);
                if k1>=0 then with Holiday[GetHolidayIndex(k1)] do begin
                  hw:=hw or (HType=htWeek);
                  if (HView<>hvHidden) and (hView<>hvNone) and (hView<>hvMoon) then k:=k1;
                  end;
                until k1<0;
              with Holiday[GetHolidayIndex(k)] do begin
                if HView=hvHoliday then begin
                  bcol:=HoCol; fcol:=HoFCol;
                  end
                else begin
                  if HView=hvPrivate then begin
                    bcol:=PrCol; fcol:=PrFCol;
                    end
                  else if dow=1 then begin
                    bcol:=SuCol; fcol:=SuFCol;
                    end
                  else if (dow=7) and cbShowSat.Checked then begin
                    bcol:=SaCol; fcol:=SaFCol;
                    end
                  else if HView=hvPrivate then begin
                    bcol:=PrCol; fcol:=PrFCol;
                    end
                  else if HView=hvOther then begin
                    bcol:=DtCol; fcol:=DtFCol;
                    end
                  else if HView=hvHidden then begin
                    bcol:=HdCol; fcol:=HdFCol;
                    end;
                  if HView=hvPrivate then hd:=cbShowPriv.Checked
                  else if HView=hvOther then hd:=cbShowDates.Checked
                  else hd:=cbShowHidden.Checked;
                  end;
                end;
              end;
            end;
          if hd then begin
            PlotRect(mw*(i-1),yd-mh*(j-1.5),mw*i,yd-mh*(j-0.5),bcol);
            s:='';
            SetTextHeight(0.2);
            if cbShowNames.Checked then s:=FHolidayCalendar.GetHolidayNames(dt,htall-[htWeek],hv);
            if hw then begin
              if length(s)>0 then s:=s+', ';
              s:=s+FHolidayCalendar.GetHolidayNames(dt,[htWeek],hv);
              end;
            if length(s)>0 then PlotColStr(mw*i-0.1,yd-mh*(j-1),s,true,alRightCenter,clBlack);
            SetTextHeight(0.3);
            end
          else begin
            if dow=1 then begin
              PlotRect(mw*(i-1),yd-mh*(j-1.5),mw*i,yd-mh*(j-0.5),SuCol);
              fcol:=SuFCol;
              end
            else if dow=2 then begin  // Kalenderwoche am Montag anzeigen
              if cbShowWeeks.Checked then
                PlotColStr(mw*i-0.1,yd-mh*(j-1),IntToStr(WeekOfTheYear(dt)),
                  true,alRightCenter,clBlack);
              end
            else if (dow=7) and cbShowSat.Checked then begin
              PlotRect(mw*(i-1),yd-mh*(j-1.5),mw*i,yd-mh*(j-0.5),SaCol);
              fcol:=SaFCol;
              end;
            end;
          PlotColStr(mw*(i-1)+xd,yd-mh*(j-1),IntToStr(DayOfTheMonth(dt)),
            true,alRightCenter,fcol);
          PlotColStr(mw*(i-1)+xd+0.1,yd-mh*(j-1),FormatSettings.ShortDayNames[dow],
            true,alLeftCenter,fcol);
          end;
        end;
      SetLineWidth(2);        // dnne Linien
      for j:=0 to 30 do begin
        CPlot (0,mh*j,3,0); CPlot (CalWidth,mh*j,2,0);
        end;
      SetLineWidth(5);        // dicke Linien
      CPlot (0,0,3,0); CPlot (0,CalHeight,2,0); CPlot (CalWidth,CalHeight,2,0);
      CPlot (CalWidth,0,2,0); CPlot (0,0,2,0);
      CPlot (0,CalHeight-MonHeight,3,0); CPlot (CalWidth,CalHeight-MonHeight,2,0);
      for i:=1 to 11 do begin
        CPlot (mw*i,0,3,0); CPlot (mw*i,CalHeight,2,0);
        end;
      end;
    bm.Free;
    end;
  end;

procedure TPrintPreviewDialog.rgLayoutClick(Sender: TObject);
begin
  if Visible then SetLayout;
  end;

procedure TPrintPreviewDialog.SetLayout;
var
  np: integer;
  po : TPrinterOrientation;
begin
  case rgLayout.ItemIndex of
  1 : np:=2;
  2 : np:=6;
  3 : np:=12;
  else np:=1;
    end;
  with rePage do begin
    MaxValue:=np;
    Value:=1;
    Visible:=np>1;
    end;
  laPage.Visible:=np>1;
  if rgLayout.ItemIndex=3 then begin
    po:=poPortrait;
    pnlWidth.Show;
    rbFull.Checked:=true;
    reWidth.Enabled:=false;
    Rand:=VRand;
    end
  else begin
    po:=poLandscape;
    pnlWidth.Hide;
    Rand:=HRand;
    end;
  if NoPrinter then begin
    if po=poLandscape then PrSize:=defLaSize else PrSize:=defPoSize;
    end
  else begin
    Printer.Orientation:=po;
    with PrSize do PvPlot.GetPaperSize (Printer,X,Y);
    end;
  with VRand do reWidth.MaxValue:=PrSize.X-Left-Right;
  NewSize;
  end;

procedure TPrintPreviewDialog.UpdateView(Sender: TObject);
begin
  pbxCal.Invalidate;
  end;

function TPrintPreviewDialog.ChangeColor (var AColor : TColor) : boolean;
begin
  with ColorDialog do begin
    Color:=AColor;
    if Execute then begin
      AColor:=Color;
      pbxCal.Invalidate;
      Result:=true;
      end
    else Result:=false;
    end;
  end;
    
procedure TPrintPreviewDialog.pnlSuColClick(Sender: TObject);
begin
  if ChangeColor(SuCol) then pnlSuCol.Color:=SuCol;
  end;

procedure TPrintPreviewDialog.pnlSaColClick(Sender: TObject);
begin
  if ChangeColor(SaCol) then pnlSaCol.Color:=SaCol;
  end;

procedure TPrintPreviewDialog.pnlHoColClick(Sender: TObject);
begin
  if ChangeColor(HoCol) then pnlHoCol.Color:=HoCol;
  end;

procedure TPrintPreviewDialog.pnlDtColClick(Sender: TObject);
begin
  if ChangeColor(DtCol) then pnlDtCol.Color:=DtCol;
  end;

procedure TPrintPreviewDialog.pnlPrColClick(Sender: TObject);
begin
  if ChangeColor(PrCol) then pnlPrCol.Color:=PrCol;
  end;

procedure TPrintPreviewDialog.pnlHdColClick(Sender: TObject);
begin
  if ChangeColor(HdCol) then pnlHdCol.Color:=HdCol;
  end;

procedure TPrintPreviewDialog.btnSuColClick(Sender: TObject);
begin
  if ChangeColor(SuFCol) then pnlSuCol.Font.Color:=SuFCol;
  end;

procedure TPrintPreviewDialog.btnSaColClick(Sender: TObject);
begin
  if ChangeColor(SaFCol) then pnlSaCol.Font.Color:=SaFCol;
  end;

procedure TPrintPreviewDialog.btnHoColClick(Sender: TObject);
begin
  if ChangeColor(HoFCol) then pnlHoCol.Font.Color:=HoFCol;
  end;

procedure TPrintPreviewDialog.bbMarginClick(Sender: TObject);
begin
  if RandDialog.Execute(CenterPos,'',Rand) then begin
    if rgLayout.ItemIndex=3 then VRand:=Rand else HRand:=Rand;
    end;
  end;

procedure TPrintPreviewDialog.btnDtColClick(Sender: TObject);
begin
  if ChangeColor(DtFCol) then pnlDtCol.Font.Color:=DtFCol;
  end;

procedure TPrintPreviewDialog.btnHdColClick(Sender: TObject);
begin
  if ChangeColor(HdFCol) then pnlHdCol.Font.Color:=HdFCol;
  end;

procedure TPrintPreviewDialog.btnPrColClick(Sender: TObject);
begin
  if ChangeColor(PrFCol) then pnlPrCol.Font.Color:=PrFCol;
  end;

procedure TPrintPreviewDialog.Execute (AYear : integer; AHolidayCalendar : THolidayCalendar);
begin
  FYear:=AYear;
  FHolidayCalendar:=AHolidayCalendar;
  ShowModal;
  end;

procedure TPrintPreviewDialog.rbFullClick(Sender: TObject);
begin
  Rand:=VRand;
  reWidth.Enabled:=false;
  NewSize;
  end;

procedure TPrintPreviewDialog.rbManualClick(Sender: TObject);
begin
  with reWidth do begin
    Enabled:=true;
    with Rand do Right:=PrSize.X-Value-Left;
    end;
  NewSize;
  end;

procedure TPrintPreviewDialog.reWidthExit(Sender: TObject);
begin
  if rbManual.Checked then begin
    with Rand do Right:=PrSize.X-reWidth.Value-Left;
    NewSize;
    end;
  end;

end.
